/* gmos - GTK Metal/Oxyde Semiconductor
 * Copyright (C) 1998 Emmanuel Rouat
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Main header file for gmos */

#ifndef __CONST_DEV_H__
#define __CONST_DEV_H__

#ifndef M_PI
#define M_PI 3.141592
#endif

#define NMOS 1
#define PMOS -1

#define LEVEL1 1
#define LEVEL2 2
#define LEVEL3 3
#define EKV 44

#define q 1.6021918e-19
#define k 1.3806226e-23
#define ni 1.45e16  /* atoms/m^3 */ 
#define Esi 11.7
#define Eox 3.9
#define E0 8.854214871e-12 /* F par m */
#define T0 273.15
#define Eg0 1.1150877 /* bandgap silicium a 300k */


enum{BLOQUED,WEAK_INV_LIN,WEAK_INV_SAT,MOD_INV_LIN,MOD_INV_SAT, \
	 LINEAR, SATURATED};


enum {C_W,C_L,C_VG,C_VD,C_VS,C_VB,C_I};


#define GLOP 1
#define PAS_GLOP 0


#define POSITIVE 1
#define NEGATIVE -1

#define MIN_V 0        /* min voltage */
#define MAX_V 10       /* max voltage */
#define MIN_L 0.8      /*  min length of a transistor */
#define MAX_L 1000     /*  max length of a transistor */
#define MIN_W 2        /*  min width of a transistor  */
#define MAX_W 1000     /*  max width of a transistor  */



#define N_A 9e99
#define NO_CONVERGENCE 6e66


/* A few useful macros */

#define min(a, b)  ((a) < (b) ? (a) : (b))
#define max(a, b)  ((a) < (b) ? (b) : (a))

#define slope(a,b) ((a) >= (b) ? -1 : 1)

/* and now for a whole bunch of defines - lets go: */

#define GMOS_VT0    1
#define GMOS_KP     2
#define GMOS_GAMMA  3
#define GMOS_PHI    4
#define GMOS_LAMBDA 5
#define GMOS_U0     6
#define GMOS_VMAX   7
#define GMOS_THETA  8
#define GMOS_UCRIT  9
#define GMOS_UEXP   10
#define GMOS_NEFF   11
#define GMOS_ETA    12
#define GMOS_KAPPA  13
#define GMOS_DELTA  14
#define GMOS_NSUB   15
#define GMOS_NSS    16
#define GMOS_NFS    17
#define GMOS_TPG    18
#define GMOS_TOX    19
#define GMOS_XJ     20
#define GMOS_COX    21
#define GMOS_LD     22
#define GMOS_WD     23
#define GMOS_TNOM   24


#define MAXLINES 256 /* max lines parsed in one single model */

#endif /* __CONST_DEV_H__ */

