/* gmos - GTK Metal/Oxyde Semiconductor
 * Copyright (C) 1998 Emmanuel Rouat
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <math.h>
#include <nan.h>
#include "gtkmain.h"
#include "defines.h"
#include "ekv.h"
#include "parsing.h"

double ekv(void)
{

    /* This is the Enz-Krummenacher-Vittoz model - a model with clear
     * analog design intentions - lots of interesting features, like 
     * continous derivatives, bulk-referenced voltages ...
     * For the moment I use the EKV-1.3 model
     */


    /* Note: equations can be arranged more efficiently 
     * but I prefer to keep a certain form to the equations
     * for the time being */

    /* FIXME  - far from finished...*/

    extern double w,l,vg,vd,vs,vb,Tc;
    extern double gm,gds,vp,vth,vdsat;
    extern int type,region,sb_tied; 
    
    extern struct ekv_tech_data n44;
    extern struct ekv_tech_data p44;
    struct ekv_tech_data t44;
    
    double cox,Phi,gamma,theta,kp,Ut,weff,leff,T;
    double nsub,u0,ueff,ucrit,ec,lambda,beta,n,tmp;
    double i=0.0,Is,If,Ir; /* beware:local variables */
    double dn_dvp,dn_dvg; /* derivatives */
    double vsb=0,vgb=0,vds=0,vdb=0;
    
    
    T=Tc+T0; 
    
    vth=N_A; /* Irrelevant in ekv */
    if(sb_tied==TRUE) vb=vs; /* source and bulk tied */

    /* Two cases: nmos and pmos */
    
    switch(type){
    case NMOS :
	
	/* All voltages are bulk-referenced */
	
	vsb=(vs)-(vb);
	vgb=(vg)-(vb);
	vdb=(vd)-(vb);
	vds=(vd)-(vs);

	t44=n44;
	break;
	    
    case PMOS :
		
	vsb=(vb)-(vs);
	vgb=(vb)-(vg);
	vdb=(vb)-(vd);
	vds=(vs)-(vd);

	t44=p44;
	break;
    }


    weff=w*1e-6;  /* Remember w and l are expressed in microns */
    leff=(l*1e-6)-2*t44.LD; 
    
    u0=t44.U0*1e-4;
    nsub=t44.NSUB*1e6;
    
    /* some fundamental parameters */
    
    cox=(E0*Eox)/(t44.TOX);
    kp=(t44.KP_defined==TRUE) ? t44.KP : u0*cox;	
    Ut=k*T/q;

    Phi=(t44.PHI_defined==TRUE) ? t44.PHI:2*Ut*(log(nsub/ni));	    
    gamma=(t44.GAMMA_defined==TRUE) ? t44.GAMMA:(sqrt(2*E0*Esi*q*nsub))/cox;
    lambda=(t44.LAMBDA_defined==TRUE) ? t44.LAMBDA:0;
    
	
    /* pinch-off voltage */
    tmp=sqrt(Phi)+.5*gamma;
    
    vp=vgb - t44.VT0 - gamma*(sqrt(vgb-t44.VT0+tmp*tmp) - tmp); 
	
    /* slope factor n and derivatives */
    
    n=1+gamma/(2*sqrt(2*Phi+vp)); /* n=dvg/dvp */
    
    dn_dvp=-0.25*gamma*pow(2*Phi+vp,-1.5);	    
    
    dn_dvg=dn_dvp/n;
    
    /* forward and reverse currents: */
	
    If=F((vp-vs)/Ut);
    Ir=F((vp-vd)/Ut);
    
    
    /* mobility reduction */
    
    theta=(t44.THETA_defined==TRUE) ? t44.THETA:0;
	
    tmp=max(vp,0);	    
    
    kp=kp/(1+theta*tmp);
    ueff=u0/(1+theta*tmp);
    
    /* velocity saturation */
    
    ucrit=(t44.UCRIT_defined==TRUE) ? t44.UCRIT:0;
    ec=ucrit*leff*0.5/Ut;
    
    tmp=sqrt(If)-3*log(If)/4;
    tmp=sqrt(1+2*tmp/ec)-1;
    
    vdsat=2*Ut*(ec*tmp+log(ec)); /* saturation voltage */
    
    tmp=max(vd,vdsat);
    
    kp=(ucrit!=0) ? kp/(1+(tmp-vs)/(leff*ucrit)) : kp;
    
    beta=kp*(weff/leff);
    
    
    /* and now the bloody current: */
    
    
    Is=2*n*beta*Ut*Ut; /* Specific current */
    
    i=Is*(If-Ir);
    
    /* FIXME : if vd=vs, we are not assured
     * that i=0 - sort of rounf-off error */
    
    /* gm=dId/dvg=dId/dvp*dvp/dvg=dId/dvp*n 
     * The following equation assumes that beta 
     * is constant in respect to vg */
    
    gm=Is*(dn_dvg*(If-Ir)/n + n*(dF_dx((vp-vs)/Ut)-dF_dx((vp-vd)/Ut)));
    
    
    /* gds=dId/vds 
     * The following equation assumes that beta,vp and n
     * are constant in respect to vds */
    
    gds=Is*(dF_dx((vp-vs)/Ut) - dF_dx((vp-vd)/Ut))/Ut;
    
    
    /* It would be interesting to have some kind of
     * criteria to determine whether we are in weak,moderate
     * or strong inversion */
    
    region=(vdb>vp) ? SATURATED:LINEAR;
    
	
    /* this is unnecessary for the moment */
    
    switch(region) { 
    case SATURATED : 
	break;		    
    case LINEAR : 
	break; 
    default:
	i=0; 
	gm=0;
	gds=0;
	break;
    }       


    
#if DEBUG   
    printf("beta=%g\nkp=%g\nGamma=%g\nPhi=%g\nlambda=%g\nvdsat=%g\n\n",beta,kp,gamma,Phi,lambda,vdsat); 

#endif

    return(i) ;

}


/* This is the interpolation function to create a continuous 
 * transition between weak and strong inversion */
extern double F(double x)
    
{
    double y;
    

    y=log(1+exp(x/2));
    y=y*y;
    
    return(y);
}


/* And its derivative */

extern double dF_dx(double x)
    
{
    double y;
    
    x=exp(x/2);
    y=log(1+x)*x;
    y=y/(1+x);
    
    return(y);
}




void parse_ekv(char * table[MAXLINES], int n)
{
    char s[255];
    char buffer[255],mos_type[]="NMOS";
   
    char seps[]=" +=\t\n";
    char seps2[]=" =\t\n";
    char *token;
    char *variable;
    
    double value;
    
    int i,type,foo,verdict;
    

    struct ekv_tech_data tmp={ /* default values */
	0,      FALSE,   /* nsub */
	0,      FALSE,   /* tox */
	0,      FALSE,   /* vt0 */
	0,      FALSE,   /* u0 */
	0,      FALSE,   /* kp */
	0,      FALSE,   /* gamma */   
	0,      FALSE,   /* lambda */
	0,      FALSE,   /* phi */
	0,      FALSE,   /* nss */
	0,      FALSE,   /* nfs */
	0,      FALSE,   /* neff */
	1,      TRUE,    /* tpg */
	0,      FALSE,   /* ld */
	0,      FALSE,   /* wd */
	0,      FALSE,   /* rs */
	0,      FALSE,   /* rd */
	27,     TRUE,    /* tnom */
	0,      FALSE,   /* vmax */
	0,      FALSE,   /* ucrit */
	0,      FALSE,   /* uexp */
	0,      FALSE,   /* delta */
	0,      FALSE    /* xj */ };

    
    extern struct ekv_tech_data n44;
    extern struct ekv_tech_data p44;

    /* first arg is mos type : */
    type=(strcmp(table[0],mos_type)==0) ? NMOS : PMOS;

#if DEBUG
    sprintf(buffer,"Received: type: %d\n",type); 
    message(buffer);
#endif
    

    for(i=1;i<n;i++){
	strcpy(s,table[i]);
	
	
	/* 
	 * ok , so we get a string of tokens - lets separate them
	 * We should have an alternance name/value 
	 */
	
	token=strtok(s,seps); 
	while(token != NULL){
		    
	    variable=token;
	    
	    token=strtok(NULL,seps2);
	    value=atof(token); 
	    
#if DEBUG
	    sprintf(buffer,"Received: %s = %g\n",variable,value); 
	    message(buffer);
#endif
	    
	    foo=affect(variable);
	    
	    switch(foo){
	    case 0 : sprintf(buffer,"WARNING:unknown variable %s\n",variable);
		message(buffer);
		break;
	    case GMOS_VT0 :tmp.VT0=value;
		tmp.VT0_defined=TRUE;
		break;
	    case GMOS_KP :tmp.KP=value;
		tmp.KP_defined=TRUE;
		break;
	    case GMOS_GAMMA :tmp.GAMMA=value;
		tmp.GAMMA_defined=TRUE;
		break;
	    case GMOS_PHI :tmp.PHI=value;
		tmp.PHI_defined=TRUE;
		break;
	    case GMOS_LAMBDA :tmp.LAMBDA=value;
		tmp.LAMBDA_defined=TRUE;
		break;
	    case GMOS_U0 :tmp.U0=value;
		tmp.U0_defined=TRUE;
		break;
	    case GMOS_VMAX :tmp.VMAX=value;
		tmp.VMAX_defined=TRUE;
		break;
	    case GMOS_UCRIT :tmp.UCRIT=value;
		tmp.UCRIT_defined=TRUE;
		break;
	    case GMOS_UEXP :tmp.UEXP=value;
		tmp.UEXP_defined=TRUE;
		break;
	    case GMOS_NEFF :tmp.NEFF=value;
		tmp.NEFF_defined=TRUE;
		break;
	    case GMOS_DELTA :tmp.DELTA=value;
		tmp.DELTA_defined=TRUE;
		break;
	    case GMOS_NSUB :tmp.NSUB=value;
		tmp.NSUB_defined=TRUE;
		break;
	    case GMOS_NSS :tmp.NSS=value;
		tmp.NSS_defined=TRUE;
		break;
	    case GMOS_NFS :tmp.NFS=value;
		tmp.NFS_defined=TRUE;
		break;
	    case GMOS_TPG :tmp.TPG=value;
		tmp.TPG_defined=TRUE;
		break;
	    case GMOS_TOX :tmp.TOX=value;
		tmp.TOX_defined=TRUE;
		break;
	    case GMOS_XJ :tmp.XJ=value;
		tmp.XJ_defined=TRUE;
		break;
	    case GMOS_LD  :tmp.LD=value;
		tmp.LD_defined=TRUE;
		break;
	    case GMOS_WD :tmp.WD=value;
		tmp.WD_defined=TRUE;
		break;
	    default :sprintf(buffer,"WARNING:irrelevant variable %s\n",variable);
		message(buffer);
	    }
	    
		    
	    token=strtok(NULL,seps2);
	    
	}

           
	    
    }
    
    /* here we should do some sort of checking to see
     * if we have all variables that we need ...
     */
    
    verdict=check_ekv(tmp);
    
    if(verdict==GLOP){	    
	if(type==NMOS) n44=tmp;
	if(type==PMOS) p44=tmp;
    }else{
	switch(verdict){
	case GMOS_NSUB : 
	    message("ERROR: NSUB not defined\n");
	    break;
	case GMOS_U0 : 
	    message("ERROR: U0 not defined\n");
	    break;
	case GMOS_TOX : 
	    message("ERROR: TOX not defined\n");
	    break;
	    
	}
	
    }

}



int check_ekv(struct ekv_tech_data l44)
{

    /* 
     * here we check the bits that are defined - we stop checking every
     * time that we find an error 
     */


    /* first, we need NSUB and U0 in all cases  AKAIK? */

    if(l44.NSUB_defined==FALSE) return(GMOS_NSUB);
    if(l44.U0_defined==FALSE) return(GMOS_U0);
    
 
    /* then we need TOX - some models make it possible to superseed
     * the value of TOX with COX, but not in this case ? 
     */

    if(l44.TOX_defined==FALSE) return(GMOS_TOX);


    return(GLOP);  /* ok */

}
