/* gmos - GTK Metal/Oxyde Semiconductor
 * Copyright (C) 1998 Emmanuel Rouat
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <math.h>
#include <nan.h>
#include "defines.h"
#include "gtkmain.h"
#include "parsing.h"
#include "loop.h"
#include "models.h"
#include "init.h"

double w,l,vg,vd,vs,vb,Tc;
double I,gm,gds,vth,vp,vdsat;
int wanted,model,type,region,sb_tied;


GtkWidget *text;
GtkWidget *w_entry,*l_entry,*vg_entry,*vd_entry,*vs_entry,*vb_entry,*T_entry;
GtkWidget *i_entry, *regime_entry,*gm_entry,*gds_entry,*vth_entry,
    *vp_entry,*vdsat_entry;





int main (int argc, char *argv[])
{
    
    static GtkWidget *window = NULL;
    GtkWidget *box2;
    GtkWidget *button;
    GtkWidget *frame;
    GtkWidget *I_box;
    GtkWidget *inputbox;
    GtkWidget *i_button;
    GtkWidget *iobox;
    GtkWidget *menu;
    GtkWidget *menubar;
    GtkWidget *menuitem;
    GtkWidget *optionmenu;
    GtkWidget *menubox;
    GtkWidget *mainbox;
    GtkWidget *outputbox;
    GtkWidget *textbox;
    GtkWidget *vscrollbar;
    
    
    GdkFont *font;
    
    
    extern GtkWidget *text;
    extern GtkWidget *w_entry,*l_entry;
    extern GtkWidget *vg_entry,*vd_entry,*vs_entry,*vb_entry,*T_entry;
    extern GtkWidget *i_entry,*regime_entry,*gm_entry,*gds_entry;
    extern GtkWidget *vth_entry,*vp_entry,*vdsat_entry;
    
    GSList *input_group;
    GSList *mos_type;
    
    
    extern double w,l,vg,vd,vs,vb,Tc;
    extern double I,gm,gds,vth,vp,vdsat;
    extern int wanted,model,type,region,sb_tied; 
    

  
    int wanted_val[] = 
    {
	C_W,
	C_L,
	C_VG,
	C_VD,
	C_VS,
	C_VB,
	C_I
    };

    int level[] =
    {
	LEVEL1,
	LEVEL2,
	EKV
    };

    init_parameters();
  

    gtk_init (&argc, &argv);
  
    /* Create our window */
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    /* gtk_window_set_policy (GTK_WINDOW(window), TRUE, TRUE, FALSE); */
    gtk_signal_connect (GTK_OBJECT (window), "delete_event",
			GTK_SIGNAL_FUNC (delete_event), NULL);
    gtk_container_border_width (GTK_CONTAINER (window), 10);
    
    /* We create a vertical box (mainbox) to pack the horizontal boxes into. */
    
    mainbox = gtk_vbox_new (FALSE, 0);

    /* now a menubar */
    get_main_menu(window, &menubar);
    gtk_box_pack_start(GTK_BOX(mainbox), menubar, FALSE, TRUE, 0);
    gtk_widget_show(menubar);    
    
    /* insert a text widget in main vertical box */
    textbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (mainbox), textbox, TRUE, TRUE, 0);
    text = gtk_text_new (NULL, NULL);
    gtk_text_set_editable (GTK_TEXT (text),FALSE); 
    gtk_box_pack_start (GTK_BOX (textbox), text, TRUE, TRUE, 0);
    gtk_text_set_word_wrap (GTK_TEXT (text), TRUE);
    gtk_text_set_point (GTK_TEXT (text), 0);
    vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
    gtk_box_pack_end (GTK_BOX (textbox), vscrollbar, FALSE, FALSE, 0);

    gtk_widget_show (vscrollbar);
    gtk_widget_show (text);
    gtk_widget_show (textbox);


  
    /* Now the transistor model selection menu*/
    
    frame = gtk_frame_new ("Mos Model");
    gtk_box_pack_start (GTK_BOX (mainbox), frame, TRUE, TRUE, 0);
    gtk_widget_show (frame);
    menubox = gtk_hbox_new (FALSE, 0);
    gtk_container_border_width (GTK_CONTAINER (menubox), 1);  
    gtk_container_add (GTK_CONTAINER (frame), menubox);
    gtk_widget_show (menubox);
  
    menu = gtk_menu_new ();

    /* FIXME: see if this can be automated */
    menuitem = gtk_menu_item_new_with_label ("LEVEL 1");
    gtk_object_set_data(GTK_OBJECT(menuitem), "level",(gpointer) level[0]);
    gtk_menu_append (GTK_MENU (menu), menuitem);
    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
			GTK_SIGNAL_FUNC (set_level_callback),
			&model); 
    gtk_widget_show (menuitem);
    menuitem = gtk_menu_item_new_with_label ("LEVEL 2");
    gtk_object_set_data(GTK_OBJECT(menuitem), "level",(gpointer) level[1]);
    gtk_menu_append (GTK_MENU (menu), menuitem);
    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
			GTK_SIGNAL_FUNC (set_level_callback),
			&model); 
    gtk_widget_show (menuitem);
    menuitem = gtk_menu_item_new_with_label ("EKV");
    gtk_object_set_data(GTK_OBJECT(menuitem), "level",(gpointer) level[2]);
    gtk_menu_append (GTK_MENU (menu), menuitem);
    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
			GTK_SIGNAL_FUNC (set_level_callback),
			&model); 
    gtk_widget_show (menuitem);
  
    optionmenu = gtk_option_menu_new ();
    gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), menu);
    gtk_box_pack_start (GTK_BOX (menubox), optionmenu, TRUE, TRUE, 0);
    gtk_widget_show (optionmenu);
  
    
    /* Now the nmos/pmos radiobuttons */
    button = gtk_radio_button_new_with_label (NULL, "Nmos");
    gtk_object_set_user_data (GTK_OBJECT (button), (gpointer) NMOS);
    gtk_box_pack_start (GTK_BOX (menubox), button, TRUE, TRUE, 0);
    gtk_widget_show (button);
    
    mos_type = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label(mos_type, "Pmos");
    gtk_object_set_user_data (GTK_OBJECT (button), (gpointer) PMOS);
    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), FALSE); 
    gtk_signal_connect(GTK_OBJECT(button), "toggled",
		       GTK_SIGNAL_FUNC(mos_type_callback),
		       &(type));
    gtk_box_pack_start (GTK_BOX (menubox), button, TRUE, TRUE, 0);
    gtk_widget_show (button);
  
  
  
    /* Now a new horizontal box, which will contain 2 vertical boxes 
       These boxes will be our input/outputs containers */
    iobox = gtk_hbox_new (TRUE, 10);
    gtk_box_pack_start (GTK_BOX (mainbox), iobox, FALSE, FALSE, 20);
  
    /******************************/
    /* First vertical box: inputs */
    /******************************/

    inputbox = gtk_vbox_new (TRUE, 5);
    gtk_box_pack_start (GTK_BOX (iobox), inputbox, FALSE, FALSE, 10);

    /* 
     * For some pervert reason, have to define the i_entry button before
     * all the others - this way it becomes the depressed button by default
     * on startup 
     */

    i_button = gtk_radio_button_new(NULL);
  
 
   /* W entry */

    input_group = gtk_radio_button_group (GTK_RADIO_BUTTON (i_button));
    button = gtk_radio_button_new(input_group);
    w_entry = gtk_entry_new();
    box2 = make_new_entry_scale(&w,0.0,501,0.1,1,1,"W()",
			      (gpointer) (wanted_val[0]),button,w_entry);


    gtk_box_pack_start (GTK_BOX (inputbox), box2, FALSE, FALSE, 0);  
    gtk_widget_show (box2);  
 
    /* L entry */

    input_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new(input_group);
    l_entry = gtk_entry_new();
    box2 = make_new_entry_scale(&l,0.1,501,0.1,1,1,"L()",
			      (gpointer) (wanted_val[1]),button,l_entry);
    gtk_box_pack_start (GTK_BOX (inputbox), box2, FALSE, FALSE, 0);  
    gtk_widget_show (box2);  

    /* VG entry */

    input_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new(input_group);
    vg_entry = gtk_entry_new();
    box2 = make_new_entry_scale(&vg,0.0,11.0,0.01,0.1,1.0,"VG",
			      (gpointer) (wanted_val[2]),button,vg_entry);
    gtk_box_pack_start (GTK_BOX (inputbox), box2, FALSE, FALSE, 0);  
    gtk_widget_show (box2);  

    /* VD entry */
 
    input_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new(input_group);
    vd_entry = gtk_entry_new();
    box2 = make_new_entry_scale(&vd,0.0,11.0,0.01,0.1,1.0,"VD",
			      (gpointer) (wanted_val[3]),button,vd_entry);
    gtk_box_pack_start (GTK_BOX (inputbox), box2, FALSE, FALSE, 0);  
    gtk_widget_show (box2);  

    /* VS entry */

    input_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new(input_group);
    vs_entry = gtk_entry_new();
    box2 = make_new_entry_scale(&vs,0.0,11.0,0.01,0.1,1.0,"VS",
			      (gpointer) (wanted_val[4]),button,vs_entry);
    gtk_box_pack_start (GTK_BOX (inputbox), box2, FALSE, FALSE, 0);  
    gtk_widget_show (box2);  

    /* VB entry */

    input_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new(input_group);
    vb_entry = gtk_entry_new();
    box2 = make_new_entry_scale(&vb,0.0,11.0,0.01,0.1,1.0,"VB",
			      (gpointer) (wanted_val[5]),button,vb_entry);
    gtk_box_pack_start (GTK_BOX (inputbox), box2, FALSE, FALSE, 0);  
    gtk_widget_show (box2);  



    /* T entry */

    T_entry = gtk_entry_new();     
    box2 = make_new_entry_scale(&Tc,-50,126,1.0,5.0,1.0,"Tc", 
				NULL,NULL,T_entry); 
    gtk_box_pack_start (GTK_BOX (inputbox), box2, FALSE, FALSE, 0);   
    gtk_widget_show (box2);   

    /********************************/
    /* Second vertical box: outputs */
    /********************************/
  
    outputbox = gtk_vbox_new (TRUE, 10);
    gtk_box_pack_end (GTK_BOX (iobox), outputbox, FALSE, FALSE, 20);

    
    i_entry = gtk_entry_new();
    I_box = make_new_output(&I," I",i_entry);    
    gtk_box_pack_start (GTK_BOX (outputbox), I_box, FALSE, FALSE, 0);
    /* input_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button)); */
    gtk_box_pack_start (GTK_BOX (I_box), i_button, TRUE, TRUE, 0);
    
    gtk_signal_connect (GTK_OBJECT(i_button), "toggled",
			GTK_SIGNAL_FUNC(set_I), i_entry);
    gtk_signal_connect(GTK_OBJECT(i_entry), "changed",  
		       GTK_SIGNAL_FUNC(enter_I_callback),&I);  
    gtk_widget_show (i_button);
    gtk_widget_show (I_box);

    regime_entry = gtk_entry_new();
    box2 = gtk_hbox_new (FALSE,0);
    gtk_entry_set_text(GTK_ENTRY(regime_entry), " "); 
    gtk_entry_set_editable (GTK_ENTRY (regime_entry), FALSE);
    gtk_box_pack_start (GTK_BOX (box2), regime_entry, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (outputbox), box2, FALSE, FALSE, 0);
    gtk_widget_show (regime_entry);
    gtk_widget_show (box2);
    
 
    gm_entry = gtk_entry_new();  
    box2 = make_new_output(&gm,"Gm",gm_entry);    
    gtk_box_pack_start (GTK_BOX (outputbox), box2, FALSE, FALSE, 0);
    gtk_widget_show (box2);  
    

    gds_entry = gtk_entry_new();  
    box2 = make_new_output(&gds,"Gds",gds_entry);    
    gtk_box_pack_start (GTK_BOX (outputbox), box2, FALSE, FALSE, 0);
    gtk_widget_show (box2);  


    vth_entry = gtk_entry_new();  
    box2 = make_new_output(&vth,"Vth",vth_entry);    
    gtk_box_pack_start (GTK_BOX (outputbox), box2, FALSE, FALSE, 0);
    gtk_widget_show (box2);
    

    vp_entry = gtk_entry_new();  
    box2 = make_new_output(&vp,"Vp",vp_entry);    
    gtk_box_pack_start (GTK_BOX (outputbox), box2, FALSE, FALSE, 0);
    gtk_widget_show (box2);


    vdsat_entry = gtk_entry_new();  
    box2 = make_new_output(&vdsat,"VDSsat",vdsat_entry);    
    gtk_box_pack_start (GTK_BOX (outputbox), box2, FALSE, FALSE, 0);
    gtk_widget_show (box2);


  
  /* pack the vbox (mainbox) which now contains all our widgets, into the
   * main window. */
  gtk_container_add (GTK_CONTAINER (window), mainbox); 
  
  
  /* and show everything left */
  gtk_widget_show (button);
  gtk_widget_show (iobox);
  gtk_widget_show (inputbox);
  gtk_widget_show (outputbox);
  
  gtk_widget_show (mainbox);
  
  /* Now we can use the text widget and display welcome message */
  font = gdk_font_load ("-*-*-medium-r-normal-*-*-*-*-*-*-100-iso8859-1");
  gtk_widget_realize(text);
  gtk_text_insert (GTK_TEXT (text), font, NULL, NULL, "Welcome To GMOS\n\n(c)E.Rouat 1998\n\n",-1); 

  /* Initialise everything */
  run_callback(NULL,NULL);
  update_outputs(NULL,NULL);
  
  /* Showing the window last so everything pops up at once. */
  gtk_widget_show (window);	
  


  /* And of course, our main function. */
  gtk_main ();
  
  /* control returns here when gtk_main_quit() is called, but not when 
   * gtk_exit is used. */
  
  return 0;
}


void init_parameters(void)
{

    extern double w,l,vg,vd,vs,vb,Tc;
    extern double I,gm,vth,vp,vdsat;
    extern int wanted,model,type,region,sb_tied; 


 
    w=10;
    l=5;
    vb=0;
    vs=1;
    vg=2.5;
    vd=2;
    Tc=27;
    model=LEVEL1;
    type=NMOS;
  
    I=0;
    gm=0;
    gds=0;
    vth=0;
    vp=0;
    vdsat=0;
    
    region=BLOQUED;
    wanted=C_I;
    sb_tied=FALSE;


}

void enter_callback(GtkWidget *widget, double *data)
{
    GtkAdjustment *adjustment;
    double new_val;
    double val;
    
    val = *data;
    new_val = atof(gtk_entry_get_text(GTK_ENTRY(widget)));
    
    if (val != new_val) {
	adjustment = gtk_object_get_data(GTK_OBJECT(widget),"adj");
	    
	if ((new_val >= adjustment->lower) &&
	    (new_val <= adjustment->upper)) {
	    val = new_val;
	    gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),new_val);
	}
    } 
}


void enter_I_callback(GtkWidget *widget, double *data)
{
    double new_val;
    
    new_val = atof(gtk_entry_get_text(GTK_ENTRY(widget)));
    *data = new_val;

    run_callback(NULL,NULL);

}


void range_callback(GtkAdjustment *adj, double *modified_variable)
{
    GtkWidget *ent;
    gchar buf[40];
    
    if(*modified_variable!=adj->value){
	    
	*modified_variable=adj->value; 
	    
	ent = gtk_object_get_user_data(GTK_OBJECT(adj)); 
	sprintf(buf, "%g", adj->value); 
	gtk_signal_handler_block_by_func(GTK_OBJECT(ent),
					 GTK_SIGNAL_FUNC(enter_callback),
					 modified_variable);
	
	gtk_entry_set_text(GTK_ENTRY(ent), buf);  
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(ent),
					   GTK_SIGNAL_FUNC(enter_callback),
					   modified_variable);
	run_callback(GTK_WIDGET(ent),NULL);
	update_outputs(NULL,NULL);
	
    }
  
}


void set_level_callback(GtkWidget *widget, int *model)
{

    *model=(int)gtk_object_get_data(GTK_OBJECT(widget),"level");
        
    run_callback(NULL,NULL);
    update_outputs(NULL,NULL);   
}


void mos_type_callback(GtkWidget *widget, int *type)
{
    if(*type==NMOS)
	*type=PMOS;
    else
	*type=NMOS;
    run_callback(NULL,NULL);
    update_outputs(NULL,NULL);   
}


void set_I (GtkWidget *button, GtkWidget *group)
{
    extern GtkWidget *i_entry;
    extern int wanted; 
    
    if (GTK_TOGGLE_BUTTON (button)->active) {
	    wanted=C_I;
	    gtk_entry_set_editable(GTK_ENTRY(i_entry),FALSE);
    }else{
	gtk_entry_set_editable(GTK_ENTRY(i_entry),TRUE);
	
    }
    
}  



void run_callback (GtkWidget *widget, int *data)
{
  
    extern int wanted,model; 
    extern GtkWidget *w_entry,*l_entry;
    extern GtkWidget *vg_entry,*vd_entry,*vs_entry,*vb_entry,*T_entry;

    GtkObject *adjustment;
    
 
    switch(wanted){
    case C_W : bissection(&w,MAX_W,MIN_W);
	adjustment = gtk_object_get_data(GTK_OBJECT(w_entry),"adj");
	gtk_signal_handler_block_by_func(GTK_OBJECT(adjustment),
					 GTK_SIGNAL_FUNC(range_callback),
					 &w);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),w);
	gtk_entry_set_text (GTK_ENTRY (w_entry),convert_to_char(w)); 
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(adjustment),
					   GTK_SIGNAL_FUNC(range_callback),
					   &w);	
	break;
    case C_L : bissection(&l,MAX_L,MIN_L);
	adjustment = gtk_object_get_data(GTK_OBJECT(l_entry),"adj");
	gtk_signal_handler_block_by_func(GTK_OBJECT(adjustment),
					 GTK_SIGNAL_FUNC(range_callback),
					 &l);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),l);
	gtk_entry_set_text (GTK_ENTRY (l_entry),convert_to_char(l)); 
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(adjustment),
					   GTK_SIGNAL_FUNC(range_callback),
					   &l);	
	break;
    case C_VG : bissection(&vg,MAX_V,MIN_V);
	adjustment = gtk_object_get_data(GTK_OBJECT(vg_entry),"adj");
	gtk_signal_handler_block_by_func(GTK_OBJECT(adjustment),
					 GTK_SIGNAL_FUNC(range_callback),
					 &vg);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),vg);
	gtk_entry_set_text (GTK_ENTRY (vg_entry),convert_to_char(vg)); 
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(adjustment),
					   GTK_SIGNAL_FUNC(range_callback),
					   &vg);	
	break;
	case C_VS : bissection(&vs,MAX_V,MIN_V);
	    adjustment = gtk_object_get_data(GTK_OBJECT(vs_entry),"adj");
	    gtk_signal_handler_block_by_func(GTK_OBJECT(adjustment),
					     GTK_SIGNAL_FUNC(range_callback),
					     &vs);
	    gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),vs);
	    gtk_entry_set_text (GTK_ENTRY (vs_entry),convert_to_char(vs)); 
	    gtk_signal_handler_unblock_by_func(GTK_OBJECT(adjustment),
					       GTK_SIGNAL_FUNC(range_callback),
					       &vs);	
	    break;
    case C_VD : bissection(&vd,MAX_V,MIN_V);
	adjustment = gtk_object_get_data(GTK_OBJECT(vd_entry),"adj");
	gtk_signal_handler_block_by_func(GTK_OBJECT(adjustment),
					 GTK_SIGNAL_FUNC(range_callback),
					 &vd);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),vd);
	gtk_entry_set_text (GTK_ENTRY (vd_entry),convert_to_char(vd)); 
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(adjustment),
					   GTK_SIGNAL_FUNC(range_callback),
					   &vd);	
	break;
    case C_VB : message("Not implemented!\n");
	break;
    case C_I : calcul_i();
	gtk_signal_handler_block_by_func(GTK_OBJECT(i_entry),
					 GTK_SIGNAL_FUNC(enter_I_callback),
					 &I);
	gtk_entry_set_text (GTK_ENTRY (i_entry),convert_to_char(I)); 
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(i_entry),
					   GTK_SIGNAL_FUNC(enter_I_callback),
					   &I);
	
	break;
	    
    default:message("Error:wanted unknown value\n");
	
    }
 
}

	    

void update_outputs(GtkWidget *widget, gpointer *data)
{
    GtkObject *adjustment;    

    extern double gm,gds,vth,vp,vdsat;
    extern int region,sb_tied; 
    
    extern GtkWidget *gm_entry,*gds_entry,*vth_entry,*vp_entry,*vdsat_entry;
    extern GtkWidget *vb_entry;
  

    gtk_entry_set_text (GTK_ENTRY (gm_entry),convert_to_char(gm));
    gtk_entry_set_text (GTK_ENTRY (gds_entry),convert_to_char(gds));
    gtk_entry_set_text (GTK_ENTRY (vth_entry),convert_to_char(vth)); 
    gtk_entry_set_text (GTK_ENTRY (vp_entry),convert_to_char(vp)); 
    gtk_entry_set_text (GTK_ENTRY (vdsat_entry),convert_to_char(vdsat));  

    switch(region){
    case LINEAR :
	output("Mos linear");
	break;
    case SATURATED :
	    output("Mos saturated");
	    break; 
    case WEAK_INV_LIN :
	output("Mos linear, weak inversion");
	break;
    case WEAK_INV_SAT :
	output("Mos saturated,weak inversion");
	break;
    default:
	output("MOS bloqued"); 
	break;
    }

    /* special case: if source and bulk are tied */

    if(sb_tied==TRUE){
	adjustment = gtk_object_get_data(GTK_OBJECT(vb_entry),"adj");
	gtk_signal_handler_block_by_func(GTK_OBJECT(adjustment),
					 GTK_SIGNAL_FUNC(range_callback),
					 &vb);
	gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),vb);
	gtk_entry_set_text (GTK_ENTRY (vb_entry),convert_to_char(vb)); 
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(adjustment),
					   GTK_SIGNAL_FUNC(range_callback),
					   &vb);	
    }
    
  
}





char *convert_to_char(double x)
{
    double tmp,mantisse;
    int exposant;
    char *c;
    static char buffer[32]; /* big enough ? */



    if((fabs(x)>=0.1&&fabs(x)<1000)||(x==0)) 
	sprintf (buffer, "%g", x);

    else{
	tmp=log10(fabs(x));
	exposant=(((int)tmp) / 3)*3;
	/* pour les nombres negatifs */
	if(exposant<=0) exposant -=3;
			    
	mantisse=x/pow(10,exposant);
	sprintf(buffer,"%ge%d\n",mantisse,exposant);
    }
    
    
    if (x==N_A) sprintf (buffer, "N/A");
    if (isnan(x)) sprintf (buffer, "N/A");
    if (x==NO_CONVERGENCE) sprintf (buffer, "No Solution?");  
    c=buffer;
       
    return c;

}


void delete_event (GtkWidget *widget, gpointer data)
{
    g_print ("%s\n", (char *) data);
    gtk_main_quit ();
}





void set_wanted_callback(GtkWidget *widget, GtkWidget *entry)
{
    extern int wanted;
    GtkObject *adjustment;
    double *var;

    adjustment = gtk_object_get_data(GTK_OBJECT(entry),"adj");
    var =  gtk_object_get_data(GTK_OBJECT(entry),"var"); 
    if (GTK_TOGGLE_BUTTON (widget)->active) {
	wanted=(int)gtk_object_get_data(GTK_OBJECT(widget),"wanted");
	gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
	gtk_signal_handler_block_by_data(GTK_OBJECT(adjustment),var); 	    
	
    }else {
	gtk_entry_set_editable(GTK_ENTRY(entry),TRUE);
	gtk_signal_handler_unblock_by_data(GTK_OBJECT(adjustment),var);
    }
    
}


void message(char *c)
{
    extern GtkWidget *text;
    
    gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL,c,-1); 
  
}

void output(char *c)
{
    extern GtkWidget *regime_entry;
    
    gtk_entry_set_text (GTK_ENTRY (regime_entry),c);  
  
}


GtkWidget *make_new_entry_scale(double *var, 
				       gfloat lower, gfloat upper,
				       gfloat step_inc, gfloat page_inc,
				       gfloat page_size,const char *label,
				       gpointer wanted,GtkWidget *button,
				       GtkWidget *entry)
{
    GtkWidget *box2;
    GtkWidget *adj;
    GtkWidget *entry_label;
    GtkObject *adjustment;
    
    box2 = gtk_hbox_new (FALSE,10);  
    adjustment = gtk_adjustment_new (*var, lower, upper,step_inc,page_inc,page_size);
    adj = gtk_hscrollbar_new (GTK_ADJUSTMENT (adjustment));
    gtk_widget_set_usize(GTK_WIDGET(adj), 120, 0); 
    gtk_box_pack_start (GTK_BOX (box2), adj, TRUE, TRUE, 0);
  
    gtk_box_pack_start(GTK_BOX(box2), entry, FALSE, TRUE, 0);
    gtk_entry_set_text(GTK_ENTRY(entry), convert_to_char(*var)); 
    gtk_object_set_data(GTK_OBJECT(entry), "adj",(gpointer) adjustment);
    gtk_object_set_data(GTK_OBJECT(entry), "var",(gpointer) var);
    gtk_object_set_user_data(GTK_OBJECT(adjustment), (gpointer) entry);
    
    gtk_signal_connect(GTK_OBJECT(adjustment), "value_changed",  
		       GTK_SIGNAL_FUNC(range_callback),var); 
    gtk_signal_connect(GTK_OBJECT(entry), "changed",  
		       GTK_SIGNAL_FUNC(enter_callback),var); 

    entry_label = gtk_label_new (label);  
    gtk_box_pack_start (GTK_BOX (box2), entry_label, TRUE, FALSE, 0);
    
    if((gpointer)(button)){ 
	gtk_object_set_data(GTK_OBJECT(button), "wanted",(gpointer) (wanted));
	gtk_box_pack_start (GTK_BOX (box2), button, TRUE, FALSE, 0);   
	gtk_signal_connect(GTK_OBJECT(button), "toggled",   
			   GTK_SIGNAL_FUNC(set_wanted_callback),entry); 
	gtk_widget_show(button);  
    }
    

    gtk_widget_show(entry);           
    gtk_widget_show(entry_label);     
    gtk_widget_show(adj);  
    
    return(box2);
    
}



GtkWidget *make_new_output(double *var,const char *label,GtkWidget *entry)
{

    GtkWidget *box2;
    GtkWidget *entry_label;


    box2 = gtk_hbox_new (FALSE,0);
    gtk_entry_set_text(GTK_ENTRY(entry), convert_to_char(*var)); 
    gtk_entry_set_editable (GTK_ENTRY (entry), FALSE);
    gtk_box_pack_start (GTK_BOX (box2), entry, TRUE, TRUE, 0);
    entry_label = gtk_label_new (label);  
    gtk_box_pack_start (GTK_BOX (box2), entry_label, TRUE, FALSE, 0);
    gtk_widget_show (entry_label);   
    gtk_widget_show (entry);

    return(box2);

}




/* 
 * Create a (sub)menu and the item that activates it.
 * Returns GtkMenu widget pointer.
 */
GtkWidget *create_menu(char *label, GtkWidget *parent)
{
	GtkWidget *item, *menu;
	
	if(label)
	    item = gtk_menu_item_new_with_label(label);
	else
	    item = gtk_menu_item_new();

	menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);

	if(parent) {
	    if(GTK_IS_MENU_BAR(parent))
		gtk_menu_bar_append (GTK_MENU_BAR (parent), item);
	    else if(GTK_IS_MENU(parent))
		gtk_menu_append(GTK_MENU(parent), item);
	}
	gtk_widget_show(item);
	return menu;
}

/*
 * helper function for making menu items.  Returns menu item widget pointer,
 * but it can often be ignored, since the item is already added to the parent.
 */
GtkWidget *create_menuitem(char *label, GtkWidget *parent, GtkSignalFunc action, 
			   gpointer p)
{
    GtkWidget *item;
    if(label)
	item = gtk_menu_item_new_with_label(label);
    else
	item = gtk_menu_item_new();
    
    if(action)
	gtk_signal_connect (GTK_OBJECT (item), "activate", action, p);
    if(parent)
	gtk_menu_append (GTK_MENU(parent), item);
    gtk_widget_show (item);
    return item;
}


static GtkMenuEntry menu_items[] =
{
    {"<Main>/File/Open", "<control>O", get_fname_load_file, NULL},
    {"<Main>/File/<separator>", NULL, NULL, NULL},
    {"<Main>/File/Quit", "<control>Q", delete_event, "OK, I'll quit"},
    {"<Main>/Options/(un)tie S to B", "<control>T", tie_source_to_bulk, NULL}
};



void get_main_menu(GtkWidget *window, GtkWidget ** menubar)
{
    int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
    GtkMenuFactory *factory;
    GtkMenuFactory *subfactory;
    
    factory = gtk_menu_factory_new(GTK_MENU_FACTORY_MENU_BAR);
    subfactory = gtk_menu_factory_new(GTK_MENU_FACTORY_MENU_BAR);
    
    gtk_menu_factory_add_subfactory(factory, subfactory, "<Main>");
    gtk_menu_factory_add_entries(factory, menu_items, nmenu_items);
    gtk_window_add_accelerator_table(GTK_WINDOW(window), subfactory->table);
    
    if (menubar)
	*menubar = subfactory->widget;
}


void tie_source_to_bulk()
{
    char buffer[255];

    extern int sb_tied;
    
    
    if(sb_tied==TRUE){
	sb_tied=FALSE;
	sprintf(buffer,"Source and bulk untied\n");
	message(buffer);
    }else{
	sb_tied=TRUE;
	sprintf(buffer,"Source and bulk tied\n");
	message(buffer);
    }
    
    /* recalculate */
    run_callback(NULL,NULL);
    
    /* and update */
    update_outputs(NULL,NULL);
    
    
}
