/* gmos - GTK Metal/Oxyde Semiconductor
 * Copyright (C) 1998 Emmanuel Rouat
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <math.h>
#include <nan.h>
#include "gtkmain.h"
#include "defines.h"
#include "level2.h"
#include "parsing.h"


double level2(void)
{
    /* Note: unfinished - lacks gds equations 
     * Rewritten using equations from Foty's book
     * See README for references 
     */


    extern double w,l,vg,vd,vs,vb,Tc;
    extern double gm,gds,vth,vp,vdsat;
    extern int type,region,sb_tied; 
    
    extern struct level2_tech_data n2;
    extern struct level2_tech_data p2;
    struct  level2_tech_data t2;
    
    double cox,Phi,Phis,Phid,gamma,gammaprim,lambda,Ut,von;
    double weff,leff,fn,nu,fs,Xd,Cd,n,lprim,beta;
    double vfb,vt0,vbi,vgn;
    double EgT,T,Tr,mut,kp;
    double uexp,ucrit,ueff,ufact;
    double Ws,Wd,argd,args,alphas,alphad;
    double dvdsat_dvgs,di_dvds,dalphad_dvds,dgammaprim_dvds;  /* derivatives */
    double xlv;
    double x1,x2; /* variables temporaires */
    double nsub,nfs;
    double i=0.0,i0,vbs=0,vgs=0,vds=0;



    vp=N_A; /* Irrelevent parameter in level2 */

  
    T=Tc+T0; 
    if(sb_tied==TRUE) vb=vs; /* source and bulk tied */


    /* Two cases: nmos and pmos */

    switch(type) {
    case NMOS :
	
	/* All voltages are source-referenced */
	
	vbs=(vb)-(vs);
	vgs=(vg)-(vs);
	vds=(vd)-(vs);
	t2=n2;
	
	break;
	
    case PMOS :

	/* All voltages are source-referenced */
    
	vbs=(vs)-(vb);
	vgs=(vs)-(vg);
	vds=(vs)-(vd);
	t2=p2;
	
	break;	
    }
    
    Tr=T/(t2.TNOM+T0); /* temperature relative */
    
    weff=(w*1e-6)-2*t2.WD;  /* Remember w and l are in microns */
    leff=(l*1e-6)-2*t2.LD; 
    
    nsub=t2.NSUB*1e6;    
    nfs=(t2.NFS_defined==TRUE) ? t2.NFS*1e4 : 0 ;
    
	
    /* some fundamental parameters */
	
    cox=(E0*Eox)/(t2.TOX);
    Ut=k*T/q;	
    Phi=Ut*(log(nsub/ni)); /* fermi potential */	
    EgT=1.16-(7.02e-4*T*T/(T+1108)); /* Eg(T) */

    Phis=2*Phi-vbs;
    Phid=2*Phi-(vb-vd);  /* = Phi - vbd */
    
    mut=t2.U0/(Tr*sqrt(Tr))*1e-4; /* (T) in m^2*/
    
    gamma=sqrt(2*E0*Esi*q*nsub)/cox;
    
    /* short channel effect */
    
    Xd=sqrt(2*E0*Esi/(q*nsub));
    Ws=Xd*sqrt(Phis);
    Wd=Xd*sqrt(Phid);
    
    if(t2.XJ_defined==TRUE) {
	argd=sqrt(1+2*Wd/t2.XJ);
	args=sqrt(1+2*Ws/t2.XJ);
	
	alphad=(t2.XJ/(2*leff))*(argd-1);
	dalphad_dvds=Xd/(4*argd*sqrt(Phid));
	
	alphas=(t2.XJ/(2*leff))*(args-1);
	fs=1-alphad-alphas; /* factor short */
    } else { 
	fs=1;
	argd=0;
	args=0;
	dalphad_dvds=0;
	
    } 
	
    gammaprim=gamma*fs; 
    dgammaprim_dvds=gamma*dalphad_dvds;
    
    vfb=(t2.TPG=0) ? 0-2*Phi-EgT/2 : 0-2*Phi-t2.TPG*EgT/2; /* flatband */
    
    vt0=(t2.VT0_defined==TRUE) ? t2.VT0 : vfb+2*Phi+gamma*sqrt(2*Phi);
    
    /* narrow channel effect */
    
    fn=(t2.DELTA_defined==TRUE) ? t2.DELTA*M_PI*E0*Esi/(4*cox*weff) : 0; /* factor narrow */
    
    nu=1+fn;
    
    /* and now the threshold voltage vth */
    
    vbi=vt0-gamma*sqrt(2*Phi) + fn*Phis;  
    vth=vbi+gammaprim*sqrt(Phis);
	    
    /* weak inversion threshold von */
    
    if(t2.XJ_defined==TRUE) 
	Cd=(gamma*Xd)/(4*leff*args) - gammaprim/(2*sqrt(Phis)) -fn;
    /* equation 6.125 in Foty is wrong */
    else
	Cd=gamma/(2*sqrt(Phis)); 
    
    
    n=1+(q*nfs/cox)+Cd;	    
    von=vth+n*Ut;
	    

    /* mobility reduction effect */
    
    ucrit=(t2.UCRIT_defined==TRUE) ? t2.UCRIT : 1e4;    
    uexp=(t2.UEXP_defined==TRUE) ? t2.UEXP*1e-4 : 0;
    ufact=ucrit*100*(Esi/Eox)*t2.TOX;	    
    ueff=((vg-von)>ufact) ? mut*pow(ufact/(vg-von),uexp) : mut;
    
    kp=ueff*cox;
    
    /* saturation voltage - grove-froman ? */
	    
    x1=(vgs-vbi)/nu;
    x2=fs*gamma/nu;
    x2=x2*x2;
    
    vdsat=x1 + (x2/2)*(1-sqrt(1+4*(x1+Phis)/x2));
    dvdsat_dvgs=1/nu * (1+1/sqrt(1+4*(x1+Phis)/x2));    
    
    /* channel length modulation */
    
    
    if(t2.LAMBDA_defined==TRUE){
	lambda=t2.LAMBDA;
	lprim=leff*lambda*vds;		    
    }else{
	/* modified Reddi-Sah  model */
	xlv=(vds-vdsat)/4;     
	lprim=Xd*sqrt(xlv + sqrt(1+xlv*xlv));			    
    }    
    
    /* 
     * Note that a full implementation of level 2 would require 
     * different equations when VMAX is defined - however, the calculations
     * involved are a real pain in the as^H^Hneck , so I chose to
     * discard them for simplicity 
     */
    
    beta=kp*weff/(leff-lprim);
    
    if(vgs<=von){ /* weak inversion*/
	region=(vds<vdsat) ? WEAK_INV_LIN : WEAK_INV_SAT;
	vgn=von-vbi;
	
    }else { /* strong inversion */
	region=(vds<vdsat) ? LINEAR : SATURATED;
	vgn=vgs-vbi;
	
    }
	    
	    
    if(vgs<=vth) region=BLOQUED; 
    
    
    switch(region){
    case LINEAR :
	x1=pow(vds+Phis,1.5);
	x2=pow(Phis,1.5);
	i=beta*(vds*(vgn-nu*vds/2)-gammaprim*(x1-x2)/1.5); 
	di_dvds=beta*(vgn-nu*vds-gammaprim*sqrt(vds+Phis));
	gm=beta*vds;
	gds=N_A; 
	break;
    case SATURATED :
	x1=pow(vdsat+Phis,1.5);
	x2=pow(Phis,1.5);
	i=beta*(vdsat*(vgn-nu*vdsat/2)-gammaprim*(x1-x2)/1.5);
	di_dvds=beta*(vgn-nu*vdsat-gammaprim*sqrt(vdsat+Phis));
	gm=beta*vdsat+di_dvds*dvdsat_dvgs;
	gds=N_A; 
	break; 
    case WEAK_INV_LIN :
	x1=pow(vds+Phis,1.5);
	x2=pow(Phis,1.5);
	i0=beta*(vds*(vgn-nu*vds/2)-gammaprim*(x1-x2)/1.5);
	i=i0*exp((vgs-von)/(n*Ut));
	gm=i/(n*Ut);
	gds=N_A;
	break;
    case WEAK_INV_SAT :
	x1=pow(vdsat+Phis,1.5);
	x2=pow(Phis,1.5);
	i0=beta*(vdsat*(vgn-nu*vdsat/2)-gammaprim*(x1-x2)/1.5);
	i=i0*exp((vgs-von)/(n*Ut));
	di_dvds=beta*(vgn-nu*vdsat-gammaprim*sqrt(vdsat+Phis));
	gm=(i0/(n*Ut)+di_dvds*dvdsat_dvgs)*exp((vgs-von)/(n*Ut));
	gds=N_A;
	break;
	
    default:
	i=0;
	gds=0; 
	break;
    }   
    
#if DEBUG

    printf("Kp=%g\nGamma=%g\nPhi=%g\nVon=%g\nlprim=%g\nleff=%g\n",kp,gammaprim,Phi,von,lprim,leff);  

#endif
   
    return i ;

}




void parse_level2(char * table[30], int n)
{
    char s[255];
    char buffer[255],mos_type[]="NMOS";
   
    char seps[]=" +=\t\n";
    char seps2[]=" =\t\n";
    char *token;
    char *variable;
    
    double value;
    
    int i,type,foo,verdict;
    

    struct level2_tech_data tmp={ /* default values */
	1,      TRUE,    /* tpg */
	0,      FALSE,   /* tox */
	0,      FALSE,   /* ld */
	0,      FALSE,   /* wd */
	0,      FALSE,   /* vt0 */
	0,      FALSE,   /* u0 */
	0,      FALSE,   /* ucrit */
	0,      FALSE,   /* uexp */
	0,      FALSE,   /* rs */
	0,      FALSE,   /* rd */
	0,      FALSE,   /* delta */
	0,      FALSE,   /* nsub */
	0,      FALSE,   /* xj */
	0,      FALSE,   /* vmax */
	0,      FALSE,   /* neff */
	0,      FALSE,   /* nfs */
	27,     TRUE     /* tnom */ };

    
    extern struct level2_tech_data n2;
    extern struct level2_tech_data p2;

    /* first arg is mos type : */
    type=(strcmp(table[0],mos_type)==0) ? NMOS : PMOS;

#if DEBUG
    sprintf(buffer,"Received: type: %d\n",type); 
    message(buffer);
#endif
    

    for(i=1;i<n;i++){
	strcpy(s,table[i]);
	
	/* 
	 * ok , so we get a string of tokens - lets separate them
	 * We should have an alternance name/value 
	 */
	
	token=strtok(s,seps); 
	while(token != NULL){
		
	    variable=token;
	    token=strtok(NULL,seps2);
	    value=atof(token); 

#if DEBUG
	    sprintf(buffer,"Received: %s = %g\n",variable,value); 
	    message(buffer);
#endif

	    foo=affect(variable);
		    
	    switch(foo){
	    case 0 :sprintf(buffer,"WARNING:unknown variable %s\n",variable);
		message(buffer);
		break;
	    case GMOS_VT0 :tmp.VT0=value;
		tmp.VT0_defined=TRUE;
		break;
	    case GMOS_U0 :tmp.U0=value;
		tmp.U0_defined=TRUE;
		break;
	    case GMOS_LAMBDA :tmp.LAMBDA=value;
		tmp.LAMBDA_defined=TRUE;
		break;
	    case GMOS_VMAX :tmp.VMAX=value;
		tmp.VMAX_defined=TRUE;
		break;
	    case GMOS_UCRIT :tmp.UCRIT=value;
		tmp.UCRIT_defined=TRUE;
		break;
	    case GMOS_UEXP :tmp.UEXP=value;
		tmp.UEXP_defined=TRUE;
		break;
	    case GMOS_NEFF :tmp.NEFF=value;
		tmp.NEFF_defined=TRUE;
		break;
	    case GMOS_DELTA :tmp.DELTA=value;
		tmp.DELTA_defined=TRUE;
		break;
	    case GMOS_NSUB :tmp.NSUB=value;
		tmp.NSUB_defined=TRUE;
		break;
	    case GMOS_NFS :tmp.NFS=value;
		tmp.NFS_defined=TRUE;
		break;
	    case GMOS_TPG :tmp.TPG=value;
		tmp.TPG_defined=TRUE;
		break;
	    case GMOS_TOX :tmp.TOX=value;
		tmp.TOX_defined=TRUE;
		break;
	    case GMOS_XJ :tmp.XJ=value;
		tmp.XJ_defined=TRUE;
		break;
	    case GMOS_LD  :tmp.LD=value;
		tmp.LD_defined=TRUE;
		break;
	    case GMOS_WD :tmp.WD=value;
		tmp.WD_defined=TRUE;
		break;
	    default :sprintf(buffer,"WARNING:irrelevant variable %s\n",variable);
		message(buffer);
	    }
	    
		    
	    token=strtok(NULL,seps2);

	}    
	    
	    
    }
    

    /* here we should do some sort of checking to see
     * if we have all variables that we need ...
     */
    
    verdict=check_level2(tmp);
    
    if(verdict==GLOP){	    
	if(type==NMOS) n2=tmp;
	if(type==PMOS) p2=tmp;
    }else{
	switch(verdict){
	case GMOS_NSUB : 
	    message("ERROR: NSUB not defined\n");
	    break;
	case GMOS_U0 : 
	    message("ERROR: U0 not defined\n");
	    break;
	case GMOS_TOX : 
	    message("ERROR: TOX not defined\n");
	    break;
	    
	}
	
    }
}



int check_level2(struct level2_tech_data l2)
{

    /* 
     * here we check the bits that are defined - we stop checking every
     * time that we find an error 
     */


    /* first, we need NSUB and U0 in all cases  AKAIK? */

    if(l2.NSUB_defined==FALSE) return(GMOS_NSUB);
    if(l2.U0_defined==FALSE) return(GMOS_U0);
    
 
    /* then we need TOX - some models make it possible to superseed
     * the value of TOX with COX, but not in this case ? 
     */

    if(l2.TOX_defined==FALSE) return(GMOS_TOX);


    return(GLOP);  /* ok */

}
