/* gmos - GTK Metal/Oxyde Semiconductor
 * Copyright (C) 1998 Emmanuel Rouat
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include <gtk/gtk.h>
#include <math.h>
#include "defines.h"
#include "gtkmain.h"
#include "loop.h"
#include "models.h" 




void calcul_i(void)
{
    extern int model;   
    extern double I;


    switch(model){
    case LEVEL1 :
	I=level1();
	break;
    case LEVEL2 :
	I=level2();
	break;
    case EKV :
	I=ekv(); 
	break;  
    default :
	message("Unknown model....\n");
	I=N_A;
	break;
    }
    
}


void bissection(double *x,double x1,double x2)
{
 
  extern double I;
  double f_int,f1,f2,x_int,eps,tmp=0;  


  /* 
   * Simple bissection method to determinate 
   * solution of eq. f(*x)-I=0 between x1 and x2
   */


  
  /* First: is there a solution? */
  *x = x1;
  f1=mos()-I;
  *x = x2;  
  f2=mos()-I;

  /* There is one if f1 and f2 
   * and of opposite sign: */

  if(f1*f2>0){
      message("\nNo solution!\n");
      *x=NO_CONVERGENCE;
  }else{
      do {
	  tmp++;	  
	  x_int=(x1+x2)/2;
	  *x = x_int;
	  f_int=mos()-I;
	  eps=(fabs(f_int/I));
	  x1=((slope(f1,f2)*f_int)>0) ? x2=x_int,x1 : x_int;	  
      }
      while(eps>1e-7 || tmp<100); /* should be enough */

#if DEBUG
      printf("x1=%g x2=%g x_int=%g f_int=%g eps=%g\n",x1,x2,x_int,f_int,eps);
#endif
      
    }

}




double mos(void)
{

    extern double gm,vth,vp,vdsat;
    extern int model;
    
    double i; /* Beware:i=local, I=global */
    
    /* on commence par foutre tout a N_A */
  
    gm=N_A;
    vp=N_A;
    vth=N_A;
    vdsat=N_A;
    
    switch(model)
	{
	case LEVEL1 :
	    i=level1();
	    break;
	case LEVEL2 :
	    i=level2();
	    break;
	case EKV :
	    i=ekv(); 
	    break;  
	default :
	    message("Unknow model....\n");
	    i=N_A;
	    break;
	}
    
    return i;
}

