/* gmos - GTK Metal/Oxyde Semiconductor
 * Copyright (C) 1998 Emmanuel Rouat
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <string.h>
#include "defines.h"
#include "gtkmain.h"
#include "parsing.h"
#include "models.h"
#include "lex.yy.h"




/*
 * put up a filename requester, get a filename
 */
void get_fname_load_file(GtkWidget *w, gpointer d)
{
    static GtkWidget *window = NULL;
    
    if (!window) {
	window = gtk_file_selection_new ("file selection dialog");
	
	gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (window));
	gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
	
	gtk_signal_connect (GTK_OBJECT (window), "destroy",
			    GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			    &window);
	gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(window)->ok_button),
			    "clicked",
			    GTK_SIGNAL_FUNC(file_selection_ok), window);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
				   "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
				   GTK_OBJECT (window));
    }
    
    if (!GTK_WIDGET_VISIBLE (window))
	gtk_widget_show (window);
    else
	gtk_widget_destroy (window);
}




/*
 * callback triggered when OK is selected in file requester.
 */
void file_selection_ok(GtkWidget  *w,GtkFileSelection *fs)
{
    
    char *fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs)); 
    parse_file(fname);
    run_callback(NULL,NULL); /* update after parsing */
    
    gtk_widget_destroy (GTK_WIDGET (fs));
    
}



/*
 * Generic function called by specific model parsers - creates
 * a hash table to allow a 'switch' statement
 */

int affect(const char *str)
    
{

    if(!strcmp(str,"VT0"))   return(GMOS_VT0);
    if(!strcmp(str,"KP"))    return(GMOS_KP);
    if(!strcmp(str,"GAMMA")) return(GMOS_GAMMA);
    if(!strcmp(str,"PHI"))   return(GMOS_PHI);
    if(!strcmp(str,"LAMBDA")) return(GMOS_LAMBDA);
    if(!strcmp(str,"UO"))    return(GMOS_U0);
    if(!strcmp(str,"VMAX"))  return(GMOS_VMAX);
    if(!strcmp(str,"THETA")) return(GMOS_THETA);
    if(!strcmp(str,"UCRIT")) return(GMOS_UCRIT);
    if(!strcmp(str,"UEXP"))  return(GMOS_UEXP);
    if(!strcmp(str,"NEFF"))  return(GMOS_NEFF);
    if(!strcmp(str,"ETA"))   return(GMOS_ETA);
    if(!strcmp(str,"KAPPA")) return(GMOS_KAPPA);
    if(!strcmp(str,"DELTA")) return(GMOS_DELTA);
    if(!strcmp(str,"NSUB"))  return(GMOS_NSUB);
    if(!strcmp(str,"NSS"))   return(GMOS_NSS);
    if(!strcmp(str,"NFS"))   return(GMOS_NFS);
    if(!strcmp(str,"TPG"))   return(GMOS_TPG);
    if(!strcmp(str,"TOX"))   return(GMOS_TOX);
    if(!strcmp(str,"XJ"))    return(GMOS_XJ);
    if(!strcmp(str,"COX"))   return(GMOS_COX);
    if(!strcmp(str,"LD"))    return(GMOS_LD);
    if(!strcmp(str,"WD"))    return(GMOS_WD);
    if(!strcmp(str,"TNOM"))  return(GMOS_TNOM);
	


    return(PAS_GLOP); /* error- unknown variable */
}



		    
